@@ -26,18 +26,24 @@
 #include <QScreen>
 #include <QSignalSpy>
 #include <QX11Info>
+
+#include <unistd.h>
+
 Q_DECLARE_METATYPE(WId)
 Q_DECLARE_METATYPE(NET::State)
 Q_DECLARE_METATYPE(NET::States)
 Q_DECLARE_METATYPE(NET::WindowType)
 Q_DECLARE_METATYPE(NET::WindowTypeMask)
 Q_DECLARE_METATYPE(NET::WindowTypes)
+Q_DECLARE_METATYPE(NET::Properties)
+Q_DECLARE_METATYPE(NET::Properties2)
 
 
 class KWindowInfoX11Test : public QObject
 {
     Q_OBJECT
 private Q_SLOTS:
+    void initTestCase();
     void init();
     void cleanup();
 
@@ -58,6 +64,8 @@ private Q_SLOTS:
     void testGroupLeader();
     void testExtendedStrut();
     void testGeometry();
+    void testDesktopFileName();
+    void testPid();
 
     // actionSupported is not tested as it's too window manager specific
     // we could write a test against KWin's behavior, but that would fail on
@@ -65,13 +73,20 @@ private Q_SLOTS:
 
 private:
     void showWidget(QWidget *widget);
-    bool waitForWindow(QSignalSpy &spy, WId winId, NET::Property property) const;
+    bool waitForWindow(QSignalSpy &spy, WId winId, NET::Properties property, NET::Properties2 properties2 = NET::Properties2()) const;
     bool verifyMinimized(WId window) const;
 
     QScopedPointer<QWidget> window;
 };
 
-bool KWindowInfoX11Test::waitForWindow(QSignalSpy& spy, WId winId, NET::Property property) const
+void KWindowInfoX11Test::initTestCase()
+{
+    QCoreApplication::setAttribute(Qt::AA_ForceRasterWidgets);
+    qRegisterMetaType<NET::Properties>();
+    qRegisterMetaType<NET::Properties2>();
+}
+
+bool KWindowInfoX11Test::waitForWindow(QSignalSpy& spy, WId winId, NET::Properties property, NET::Properties2 property2) const
 {
     // we need to wait, window manager has to react and update the property.
     bool foundOurWindow = false;
@@ -84,8 +99,15 @@ bool KWindowInfoX11Test::waitForWindow(QSignalSpy& spy, WId winId, NET::Property
             if (it->first().value<WId>() != winId) {
                 continue;
             }
-            if (it->last().toUInt() != property) {
-                continue;
+            if (property != NET::Properties()) {
+                if (it->at(1).value<NET::Properties>() != property) {
+                    continue;
+                }
+            }
+            if (property2 != NET::Properties2()) {
+                if (it->at(2).value<NET::Properties2>() != property2) {
+                    continue;
+                }
             }
             foundOurWindow = true;
             break;
@@ -195,7 +217,8 @@ void KWindowInfoX11Test::testState()
         QVERIFY(!info.hasState(NET::States(1 << i)));
     }
 
-    QSignalSpy spy(KWindowSystem::self(), SIGNAL(windowChanged(WId,unsigned int)));
+    QSignalSpy spy(KWindowSystem::self(), static_cast<void (KWindowSystem::*)(WId,NET::Properties,NET::Properties2)>(&KWindowSystem::windowChanged));
+    QVERIFY(spy.isValid());
     // now we have a clean window and can do fun stuff
     KWindowSystem::setState(window->winId(), state);
 
@@ -244,7 +267,8 @@ void KWindowInfoX11Test::testDemandsAttention()
     QVERIFY(info.valid());
     QVERIFY(!info.hasState(NET::DemandsAttention));
 
-    QSignalSpy spy(KWindowSystem::self(), SIGNAL(windowChanged(WId,unsigned int)));
+    QSignalSpy spy(KWindowSystem::self(), static_cast<void (KWindowSystem::*)(WId,NET::Properties,NET::Properties2)>(&KWindowSystem::windowChanged));
+    QVERIFY(spy.isValid());
     // now we have a clean window and can do fun stuff
     KWindowSystem::demandAttention(window->winId());
 
@@ -385,6 +409,7 @@ void KWindowInfoX11Test::testWindowType_data()
     QTest::newRow("notification")       << NET::NotificationMask << NET::Notification << NET::Notification;
     QTest::newRow("ComboBox")           << NET::ComboBoxMask     << NET::ComboBox     << NET::ComboBox;
     QTest::newRow("DNDIcon")            << NET::DNDIconMask      << NET::DNDIcon      << NET::DNDIcon;
+    QTest::newRow("OnScreenDisplay")    << NET::OnScreenDisplayMask << NET::OnScreenDisplay << NET::OnScreenDisplay;
 
     // incorrect masks
     QTest::newRow("desktop-unknown")      << NET::NormalMask << NET::Desktop      << NET::Unknown;
@@ -402,6 +427,7 @@ void KWindowInfoX11Test::testWindowType_data()
     QTest::newRow("notification-unknown") << NET::NormalMask << NET::Notification << NET::Unknown;
     QTest::newRow("ComboBox-unknown")     << NET::NormalMask << NET::ComboBox     << NET::Unknown;
     QTest::newRow("DNDIcon-unknown")      << NET::NormalMask << NET::DNDIcon      << NET::Unknown;
+    QTest::newRow("OnScreenDisplay-unknown") << NET::NormalMask << NET::OnScreenDisplay << NET::Unknown;
 }
 
 void KWindowInfoX11Test::testWindowType()
@@ -438,7 +464,8 @@ void KWindowInfoX11Test::testDesktop()
     }
 
     // set on all desktop
-    QSignalSpy spy(KWindowSystem::self(), SIGNAL(windowChanged(WId,unsigned int)));
+    QSignalSpy spy(KWindowSystem::self(), static_cast<void (KWindowSystem::*)(WId,NET::Properties,NET::Properties2)>(&KWindowSystem::windowChanged));
+    QVERIFY(spy.isValid());
     KWindowSystem::setOnAllDesktops(window->winId(), true);
     QVERIFY(waitForWindow(spy, window->winId(), NET::WMDesktop));
 
@@ -472,11 +499,11 @@ void KWindowInfoX11Test::testDesktop()
 void KWindowInfoX11Test::testActivities()
 {
     NETRootInfo rootInfo(QX11Info::connection(), NET::Supported | NET::SupportingWMCheck);
-    qRegisterMetaType<unsigned int>("NET::Properties");
-    qRegisterMetaType<unsigned int>("NET::Properties2");
-    QSignalSpy spyReal(KWindowSystem::self(), SIGNAL(windowChanged(WId,NET::Properties,NET::Properties2)));
+    QSignalSpy spyReal(KWindowSystem::self(), static_cast<void (KWindowSystem::*)(WId,NET::Properties,NET::Properties2)>(&KWindowSystem::windowChanged));
+    QVERIFY(spyReal.isValid());
 
     KWindowInfo info(window->winId(), 0, NET::WM2Activities);
+    QVERIFY(info.valid());
 
     QStringList startingActivities = info.activities();
 
@@ -488,7 +515,7 @@ void KWindowInfoX11Test::testActivities()
     // Window on all activities
     KWindowSystem::self()->setOnActivities(window->winId(), QStringList());
 
-    QVERIFY(waitForWindow(spyReal, window->winId(), (NET::Property)NET::WM2Activities));
+    QVERIFY(waitForWindow(spyReal, window->winId(), NET::Properties(), NET::WM2Activities));
 
     KWindowInfo info2(window->winId(), 0, NET::WM2Activities);
 
@@ -497,7 +524,7 @@ void KWindowInfoX11Test::testActivities()
     // Window on a specific activity
     KWindowSystem::self()->setOnActivities(window->winId(), QStringList() << "test-activity");
 
-    QVERIFY(waitForWindow(spyReal, window->winId(), (NET::Property)NET::WM2Activities));
+    QVERIFY(waitForWindow(spyReal, window->winId(), NET::Properties(), NET::WM2Activities));
 
     KWindowInfo info3(window->winId(), 0, NET::WM2Activities);
 
@@ -507,7 +534,7 @@ void KWindowInfoX11Test::testActivities()
     // Window on a two activities
     KWindowSystem::self()->setOnActivities(window->winId(), QStringList() << "test-activity" << "test-activity2");
 
-    QVERIFY(waitForWindow(spyReal, window->winId(), (NET::Property)NET::WM2Activities));
+    QVERIFY(waitForWindow(spyReal, window->winId(), NET::Properties(), NET::WM2Activities));
 
     KWindowInfo info4(window->winId(), 0, NET::WM2Activities);
 
@@ -518,7 +545,7 @@ void KWindowInfoX11Test::testActivities()
     // Window on the starting activity
     KWindowSystem::self()->setOnActivities(window->winId(), startingActivities);
 
-    QVERIFY(waitForWindow(spyReal, window->winId(), NET::Property(NET::WM2Activities)));
+    QVERIFY(waitForWindow(spyReal, window->winId(), NET::Properties(), NET::WM2Activities));
 
     KWindowInfo info5(window->winId(), 0, NET::WM2Activities);
 
@@ -694,10 +721,11 @@ void KWindowInfoX11Test::testExtendedStrut()
 void KWindowInfoX11Test::testGeometry()
 {
     KWindowInfo info(window->winId(), NET::WMGeometry | NET::WMFrameExtents);
-    QCOMPARE(info.geometry(), window->geometry());
-    QCOMPARE(info.frameGeometry(), window->frameGeometry());
+    QCOMPARE(info.geometry().size(), window->geometry().size());
+    QCOMPARE(info.frameGeometry().size(), window->frameGeometry().size());
 
-    QSignalSpy spy(KWindowSystem::self(), SIGNAL(windowChanged(WId,unsigned int)));
+    QSignalSpy spy(KWindowSystem::self(), static_cast<void (KWindowSystem::*)(WId,NET::Properties,NET::Properties2)>(&KWindowSystem::windowChanged));
+    QVERIFY(spy.isValid());
 
     // this is tricky, KWin is smart and doesn't allow all geometries we pass in
     // setting to center of screen should work, though
@@ -712,6 +740,38 @@ void KWindowInfoX11Test::testGeometry()
     QCOMPARE(info2.frameGeometry(), window->frameGeometry());
 }
 
+void KWindowInfoX11Test::testDesktopFileName()
+{
+    KWindowInfo info(window->winId(), NET::Properties(), NET::WM2DesktopFileName);
+    QVERIFY(info.valid());
+    QCOMPARE(info.desktopFileName(), QByteArray());
+
+    QSignalSpy spy(KWindowSystem::self(), static_cast<void (KWindowSystem::*)(WId,NET::Properties,NET::Properties2)>(&KWindowSystem::windowChanged));
+    QVERIFY(spy.isValid());
+
+    // create a NETWinInfo to set the desktop file name
+    NETWinInfo netInfo(QX11Info::connection(), window->winId(), QX11Info::appRootWindow(), NET::Properties(), NET::Properties2());
+    netInfo.setDesktopFileName("org.kde.foo");
+    xcb_flush(QX11Info::connection());
+
+    // it's just a property change so we can easily refresh
+    QX11Info::getTimestamp();
+    QTRY_COMPARE(spy.count(), 1);
+    QCOMPARE(spy.first().at(0).value<WId>(), window->winId());
+    QCOMPARE(spy.first().at(2).value<NET::Properties2>(), NET::Properties2(NET::WM2DesktopFileName));
+
+    KWindowInfo info2(window->winId(), NET::Properties(), NET::WM2DesktopFileName);
+    QVERIFY(info2.valid());
+    QCOMPARE(info2.desktopFileName(), QByteArrayLiteral("org.kde.foo"));
+}
+
+void KWindowInfoX11Test::testPid()
+{
+    KWindowInfo info(window->winId(), NET::WMPid);
+    QVERIFY(info.valid());
+    QCOMPARE(info.pid(), getpid());
+}
+
 QTEST_MAIN(KWindowInfoX11Test)
 
 #include "kwindowinfox11test.moc"
